@@ -221,6 +221,7 @@ config.JobSubmitter.skipRefreshCount = 20  # (If above the threshold meet, cache
 config.JobSubmitter.submitScript = os.path.join(os.environ["WMCORE_ROOT"], submitScript)
 config.JobSubmitter.extraMemoryPerCore = 500  # in MB
 config.JobSubmitter.drainGraceTime = 2 * 24 * 60 * 60  # in seconds
+config.JobSubmitter.useOauthToken = False
 
 config.component_("JobTracker")
 config.JobTracker.namespace = "WMComponent.JobTracker.JobTracker"
@@ -352,8 +353,8 @@ config.AgentStatusWatcher.resourceUpdaterPollInterval = 900  # [second]
 config.AgentStatusWatcher.grafanaURL = "https://monit-grafana.cern.ch"
 config.AgentStatusWatcher.grafanaToken = "OVERWRITE_BY_SECRETS"
 config.AgentStatusWatcher.grafanaSSB = 9475  # monit-grafana API number for Site Status Board
-config.AgentStatusWatcher.pendingSlotsSitePercent = 75  # [percent] Pending slots percent over site max running for a site
-config.AgentStatusWatcher.pendingSlotsTaskPercent = 70  # [percent] Pending slots percent over task max running for tasks
+config.AgentStatusWatcher.pendingSlotsSitePercent = 50  # [percent] Pending slots percent over site max running for a site
+config.AgentStatusWatcher.pendingSlotsTaskPercent = 45  # [percent] Pending slots percent over task max running for tasks
 config.AgentStatusWatcher.runningExpressPercent = 30  # [percent] Only used for tier0 agent
 config.AgentStatusWatcher.runningRepackPercent = 10  # [percent] Only used for tier0 agent
 config.AgentStatusWatcher.t1SitesCores = 30  # [percent] Only used for tier0 agent
@@ -378,7 +379,7 @@ config.RucioInjector.cacheExpiration = 2 * 24 * 60 * 60  # two days
 config.RucioInjector.createBlockRules = True
 config.RucioInjector.RSEPostfix = False  # enable it to append _Test to the RSE names
 config.RucioInjector.metaDIDProject = "Production"
-config.RucioInjector.containerDiskRuleParams = {"weight": "ddm_quota", "copies": 2, "grouping": "DATASET"}
+config.RucioInjector.containerDiskRuleParams = {"weight": "dm_weight", "copies": 2, "grouping": "DATASET"}
 config.RucioInjector.blockRuleParams = {}
 # this RSEExpr below might be updated by wmagent-mod-config script
 config.RucioInjector.containerDiskRuleRSEExpr = "(tier=2|tier=1)&cms_type=real&rse_type=DISK"
