@@ -50,6 +50,7 @@ from WMCore.DAOFactory import DAOFactory
 from WMCore.Services.UUIDLib import makeUUID
 from WMCore.Services.WMStatsServer.WMStatsServer import WMStatsServer
 from WMCore.Services.DBS.DBSErrors import DBSError
+from WMCore.Services.DBS.DBSUtils import DBSErrors
 from WMCore.WMException import WMException
 from WMCore.WorkerThreads.BaseWorkerThread import BaseWorkerThread
 
@@ -62,7 +63,7 @@ def uploadWorker(workInput, results, dbsUrl, gzipEncoding=False):
     Get confirmation in the output
 
     :param workInput: work input data
-    :param results: output results dictionary
+    :param results: multiprocessing.Queue object we we can store and retrieve dict objects
     :param dbsUrl: url of DBS server to use
     :param gzipEncoding: specify if we should use gzipEncoding
     """
@@ -70,6 +71,7 @@ def uploadWorker(workInput, results, dbsUrl, gzipEncoding=False):
     # Init DBS Stuff
     logging.debug("Creating dbsAPI with address %s", dbsUrl)
     dbsApi = DbsApi(url=dbsUrl, useGzip=gzipEncoding)
+    dbsErrors = DBSErrors(dbsUrl)
 
     while True:
 
@@ -90,40 +92,32 @@ def uploadWorker(workInput, results, dbsUrl, gzipEncoding=False):
 
         # Do stuff with DBS
         try:
+            # check if block exists in DBS
+            records = dbsApi.listBlocks(block_name=name)
+            # the records are shown in the following form
+            # [{'block_name': '/block/name#123'}]
+            if len(records) == 1 and records[0]['block_name'] == name:
+                # found that we have this block, i.e. no need to insert block anymore
+                logging.info("block %s already exist in DBS", name)
+                results.put({'name': name, 'success': "uploaded"})
+                continue
             logging.info("About to call insert block for: %s", name)
             dbsApi.insertBulkBlock(blockDump=block)
             results.put({'name': name, 'success': "uploaded"})
         except HTTPError as ex:
-            # when we will have new dbs3-client and fine granularity DBS Go server
-            # error codes then we should remove if/elif/else and replace it with
-            # DBSError(ex.body), e.g.
-            # dbsError = DBSError(ex.body)
-            # reason = dbsError.getReason()
-            # srvCode = dbsError.getServverCode()
-            # results.put({'name': name, 'success': 'error', 'error': reason, 'server_code': srvCode})
-
-            # legacy code which deal with DBS python server exceptions
+            # DBS Go server errors are defined here:
+            # https://github.com/dmwm/dbs2go/blob/master/dbs/errors.go
             dbsError = DBSError(ex.body)
-            reason = dbsError.getReason()
-            if 'Block %s already exists' % name in reason:
-                # Then this is probably a duplicate
-                # Ignore this for now
-                logging.warning("Block %s already exists. Marking it as uploaded.", name)
-                logging.debug("Exception: %s", reason)
-                results.put({'name': name, 'success': "uploaded"})
-            # python server error produces Missing message, while
-            # Go based server produces unable to find dataset_id message
-            elif 'Missing data when inserting to dataset_parents' in reason or \
-                'unable to find dataset_id' in reason:
-                msg = "Parent dataset is not inserted yet for block %s." % name
-                logging.warning(msg)
-                results.put({'name': name, 'success': "error", 'error': msg})
-            else:
-                msg = f"Error trying to process block {name} through DBS. Reason: {reason}"
-                logging.error(msg)
-                results.put({'name': name, 'success': "error", 'error': msg})
+            message = dbsError.getMessage()
+            srvCode = dbsError.getServerCode()
+            meaning = dbsErrors.get(srvCode, 'DBS server does not provide code explanation')
+            msg = f'Error trying to process block {name} through DBS. Details:'
+            msg += f'\nDBSError code     : {srvCode} ({meaning})'
+            msg += f'\nDBS server message: {message}'
+            logging.error(msg)
+            results.put({'name': name, 'success': "error", 'error': msg})
         except Exception as ex:
-            msg = f"Hit a general exception while inserting block {name. Error: {str(ex}}"
+            msg = f"Hit a general exception while inserting block {name}. Error: {str(ex)}"
             logging.exception(msg)
             results.put({'name': name, 'success': "error", 'error': msg})
     return
@@ -198,6 +192,10 @@ class DBSUploadPoller(BaseWorkerThread):
         self.physicsGroup = getattr(self.config.DBS3Upload, "physicsGroup", "NoGroup")
         self.datasetType = getattr(self.config.DBS3Upload, "datasetType", "PRODUCTION")
         self.primaryDatasetType = getattr(self.config.DBS3Upload, "primaryDatasetType", "mc")
+        self.uploaderName = getattr(self.config.DBS3Upload, "uploaderName", "WMAgent")
+        if self.uploaderName not in ("WMAgent", "T0Prod", "T0Replay"):
+            raise DBSUploadException(f"Invalid value for attribute uploaderName: {self.uploaderName}")
+
         self.blockCount = 0
         self.gzipEncoding = getattr(self.config.DBS3Upload, 'gzipEncoding', False)
         self.dbsApi = DbsApi(url=self.dbsUrl)
@@ -213,7 +211,7 @@ class DBSUploadPoller(BaseWorkerThread):
 
         self.filesToUpdate = []
 
-        self.produceCopy = getattr(self.config.DBS3Upload, 'dumpBlock', False)
+        self.dumpBlockJsonFor = getattr(self.config.DBS3Upload, 'dumpBlockJsonFor', "")
 
         self.copyPath = os.path.join(getattr(self.config.DBS3Upload, 'componentDir', '/data/srv/'),
                                      'dbsuploader_block.json')
@@ -365,6 +363,7 @@ class DBSUploadPoller(BaseWorkerThread):
 
         Find all blocks; make sure they're in the cache
         """
+        logging.info("Executing loadBlocks method...")
         openBlocks = self.dbsUtil.findOpenBlocks()
         logging.info("Found %d open blocks.", len(openBlocks))
         logging.debug("These are the openblocks: %s", openBlocks)
@@ -377,6 +376,7 @@ class DBSUploadPoller(BaseWorkerThread):
 
         # Now load the blocks
         try:
+            logging.info("Now loading %d open blocks not yet cached...", len(blocksToLoad))
             loadedBlocks = self.dbsUtil.loadBlocks(blocksToLoad)
             logging.info("Loaded %d blocks from the database.", len(loadedBlocks))
         except WMException:
@@ -389,9 +389,11 @@ class DBSUploadPoller(BaseWorkerThread):
             raise DBSUploadException(msg) from None
 
         for blockInfo in loadedBlocks:
+            logging.info("Creating DBSBufferBlock object for: %s", blockInfo['block_name'])
             block = DBSBufferBlock(name=blockInfo['block_name'],
                                    location=blockInfo['origin_site_name'],
-                                   datasetpath=blockInfo['datasetpath'])
+                                   datasetpath=blockInfo['datasetpath'],
+                                   uploader=self.uploaderName)
 
             parent = self.datasetParentageCache.get(blockInfo['datasetpath'])
             if parent:
@@ -432,6 +434,7 @@ class DBSUploadPoller(BaseWorkerThread):
         Load all files that need to be loaded.  I will do this by DatasetPath
         to break the monstrous calls down into smaller chunks.
         """
+        logging.info("Executing loadFiles method...")
         dspList = self.dbsUtil.findUploadableDAS()
 
         readyBlocks = []
@@ -501,6 +504,7 @@ class DBSUploadPoller(BaseWorkerThread):
 
         Mark Open blocks as Pending if they have timed out or their workflows have completed
         """
+        logging.info("Executing checkBlockCompletion method...")
         completedWorkflows = self.dbsUtil.getCompletedWorkflows()
         for block in viewvalues(self.blockCache):
             if block.status == "Open":
@@ -561,7 +565,8 @@ class DBSUploadPoller(BaseWorkerThread):
         blockname = "%s#%s" % (datasetpath, makeUUID())
         newBlock = DBSBufferBlock(name=blockname,
                                   location=location,
-                                  datasetpath=datasetpath)
+                                  datasetpath=datasetpath,
+                                  uploader=self.uploaderName)
         # Now we load the dataset information
         self.setDatasetInfo(newBlock)
 
@@ -609,6 +614,7 @@ class DBSUploadPoller(BaseWorkerThread):
          Open, in DBSBuffer - Newly created block that has already been
            written to DBSBuffer.  We don't have to do anything with it.
         """
+        logging.info("Executing inputBlocks method...")
         if not self.blockCache:
             return
 
@@ -641,10 +647,12 @@ class DBSUploadPoller(BaseWorkerThread):
             try:
                 myThread.transaction.begin()
                 if createInDBSBuffer:
+                    logging.info("Creating %d new blocks in DBSBuffer", len(createInDBSBuffer))
                     self.createBlocksDAO.execute(blocks=createInDBSBuffer,
                                                  conn=myThread.transaction.conn,
                                                  transaction=True)
                 if updateInDBSBuffer:
+                    logging.info("Updating %d blocks in DBSBuffer", len(updateInDBSBuffer))
                     self.updateBlocksDAO.execute(blocks=updateInDBSBuffer,
                                                  conn=myThread.transaction.conn,
                                                  transaction=True)
@@ -671,6 +679,7 @@ class DBSUploadPoller(BaseWorkerThread):
         if self.filesToUpdate:
             try:
                 myThread.transaction.begin()
+                logging.info("Associating %d files to blocks in DBSBuffer", len(self.filesToUpdate))
                 self.setBlockFilesDAO.execute(binds=self.filesToUpdate,
                                               conn=myThread.transaction.conn,
                                               transaction=True)
@@ -710,7 +719,8 @@ class DBSUploadPoller(BaseWorkerThread):
             logging.info("Queueing block for insertion: %s", block.getName())
             self.workInput.put({'name': block.getName(), 'block': encodedBlock})
             self.blockCount += 1
-            if self.produceCopy:
+            if self.dumpBlockJsonFor and (self.dumpBlockJsonFor == block.getName()):
+                logging.info("Dumping '%s' information into %s", block.getName(), self.copyPath)
                 with open(self.copyPath, 'w') as jo:
                     json.dump(encodedBlock, jo, indent=2)
             self.queuedBlocks.append(block.getName())
@@ -728,6 +738,7 @@ class DBSUploadPoller(BaseWorkerThread):
 
         To do this, the result queue needs to pass back the blockname
         """
+        logging.info("Executing retrieveBlocks method...")
         myThread = threading.currentThread()
 
         blocksToClose = []
@@ -774,18 +785,15 @@ class DBSUploadPoller(BaseWorkerThread):
             elif result["success"] == "check":
                 block = result["name"]
                 self.blocksToCheck.append(block)
-            else:
-                logging.error("Error found in multiprocess during process of block %s", result.get('name'))
-                logging.error(result['error'])
-                # Continue to the next block
-                # Block will remain in pending status until it is transferred
 
         if loadedBlocks:
             try:
+                logging.info("Updating database record with files injected into DBS")
                 myThread.transaction.begin()
                 self.updateFilesDAO.execute(blocks=loadedBlocks, status="InDBS",
                                             conn=myThread.transaction.conn,
                                             transaction=True)
+                logging.info("Updating %d blocks successfully injected into DBS", len(loadedBlocks))
                 self.updateBlocksDAO.execute(blocks=loadedBlocks,
                                              conn=myThread.transaction.conn,
                                              transaction=True)
@@ -834,6 +842,7 @@ class DBSUploadPoller(BaseWorkerThread):
         Check with DBS3 if the blocks marked as check are
         uploaded or not.
         """
+        logging.info("Checking for blocks potentially uploaded but not marked as such in the database.")
         myThread = threading.currentThread()
 
         blocksUploaded = []
@@ -843,12 +852,14 @@ class DBSUploadPoller(BaseWorkerThread):
             logging.debug("Checking block existence: %s", block)
             # Check in DBS if the block was really inserted
             try:
+                # FIXME: is it still an empty list for blocks not available?
                 result = self.dbsApi.listBlocks(block_name=block)
                 # it is an empty list if block cannot be found
                 if result:
                     loadedBlock = self.blockCache.get(block)
                     loadedBlock.status = 'InDBS'
                     blocksUploaded.append(loadedBlock)
+                    logging.info("Block '%s' will be marked as '%s'", block, loadedBlock.status)
             except Exception as ex:
                 msg = "Error trying to check block %s through DBS. Error: %s" % (block, str(ex))
                 logging.exception(msg)
@@ -856,6 +867,7 @@ class DBSUploadPoller(BaseWorkerThread):
         # Update the status of those blocks that were truly inserted
         if blocksUploaded:
             try:
+                logging.info("Marking bulk of blocks as 'InDBS'")
                 myThread.transaction.begin()
                 self.updateBlocksDAO.execute(blocks=blocksUploaded,
                                              conn=myThread.transaction.conn,
@@ -875,6 +887,8 @@ class DBSUploadPoller(BaseWorkerThread):
                 raise DBSUploadException(msg) from None
             else:
                 myThread.transaction.commit()
+                logging.info("A total of %d blocks have been successfully updated to 'InDBS'",
+                             len(blocksUploaded))
 
         for block in blocksUploaded:
             # Clean things up
