@@ -20,7 +20,7 @@ from WMCore.DAOFactory import DAOFactory
 from WMCore.FwkJobReport.Report import Report
 from WMCore.WMInit import getWMBASE
 from WMCore.Lexicon import getIterMatchObjectOnRegexp, WMEXCEPTION_REGEXP, CONDOR_LOG_FILTER_REGEXP
-
+from WMCore.Services.TagCollector.TagCollector import TagCollector
 
 def activityToType(jobActivity):
     """
@@ -138,6 +138,10 @@ class SimpleCondorPlugin(BasePlugin):
         # These are added now by the condor client
         #self.x509userproxysubject = proxy.getSubject()
         #self.x509userproxyfqan = proxy.getAttributeFromProxy(self.x509userproxy)
+        
+        self.tc = TagCollector()
+
+        self.useCMSToken = getattr(config.JobSubmitter, 'useOauthToken', False)
 
         return
 
@@ -163,9 +167,8 @@ class SimpleCondorPlugin(BasePlugin):
 
             logging.debug("Start: Submitting %d jobs using Condor Python Submit", len(jobParams))
             try:
-                with schedd.transaction() as txn:
-                    submitRes = sub.queue_with_itemdata(txn, 1, iter(jobParams))
-                    clusterId = submitRes.cluster()
+                submitRes = schedd.submit(sub, itemdata=iter(jobParams))
+                clusterId = submitRes.cluster()
             except Exception as ex:
                 logging.error("SimpleCondorPlugin job submission failed.")
                 logging.exception(str(ex))
@@ -500,6 +503,8 @@ class SimpleCondorPlugin(BasePlugin):
 
         undefined = 'UNDEFINED'
         jobParameters = []
+        # fetch an up-to-date list of CMSSW micro-architectures
+        rel_microarchs = self.tc.defaultMicroArchVersionNumberByRelease()
 
         for job in jobList:
             ad = {}
@@ -520,6 +525,11 @@ class SimpleCondorPlugin(BasePlugin):
                 ad['Requirements'] = self.reqStr
 
             ad['My.x509userproxy'] = classad.quote(self.x509userproxy)
+
+            # Allow oauth based token authentication
+            if self.useCMSToken:
+                ad['use_oauth_services'] = "cms"
+
             sites = ','.join(sorted(job.get('possibleSites')))
             ad['My.DESIRED_Sites'] = classad.quote(str(sites))
             sites = ','.join(sorted(job.get('potentialSites')))
@@ -567,19 +577,31 @@ class SimpleCondorPlugin(BasePlugin):
             # while we do not support a third option for RequiresGPU, make a binary decision
             if job['requiresGPU'] == "required":
                 ad['My.RequiresGPU'] = "1"
-                ad['request_GPUs'] = "1"
+                ad['request_gpus'] = "1"
             else:
                 ad['My.RequiresGPU'] = "0"
-                ad['request_GPUs'] = "0"
+                ad['request_gpus'] = "0"
             if job.get('gpuRequirements', None):
-                ad['My.GPUMemoryMB'] = str(job['gpuRequirements']['GPUMemoryMB'])
-                cudaCapabilities = ','.join(sorted(job['gpuRequirements']['CUDACapabilities']))
-                ad['My.CUDACapability'] = classad.quote(str(cudaCapabilities))
-                ad['My.CUDARuntime'] = classad.quote(job['gpuRequirements']['CUDARuntime'])
+                ad['My.DESIRED_GPUMemoryMB'] = str(job['gpuRequirements']['GPUMemoryMB'])
+                # CUDACapabilities is a list of strings, with each string matching this regex: r"^\d+.\d$"
+                # E.g.: ["1.0", "10.0", "2.1"]
+                cudaCapabilities = sorted(job['gpuRequirements']['CUDACapabilities'], key=float)
+                
+                ad['My.DESIRED_GPUMinimumCapability'] = classad.quote(str(cudaCapabilities[0]))
+                ad['My.DESIRED_GPUMaximumCapability'] = classad.quote(str(cudaCapabilities[-1]))
+                ad['My.DESIRED_GPURuntime'] = classad.quote(job['gpuRequirements']['CUDARuntime'])
             else:
-                ad['My.GPUMemoryMB'] = undefined
-                ad['My.CUDACapability'] = undefined
-                ad['My.CUDARuntime'] = undefined
+                ad['My.DESIRED_GPUMemoryMB'] = undefined
+                ad['My.DESIRED_GPUMinimumCapability'] = undefined
+                ad['My.DESIRED_GPUMaximumCapability'] = undefined
+                ad['My.DESIRED_GPURuntime'] = undefined
+
+            # Assign GPU related HTCondor macros classads to DESIRED classads above
+            ad['gpus_minimum_memory'] = ad['My.DESIRED_GPUMemoryMB']
+            ad['gpus_minimum_capability'] = ad['My.DESIRED_GPUMinimumCapability']
+            ad['gpus_maximum_capability'] = ad['My.DESIRED_GPUMaximumCapability']
+            ad['gpus_minimum_runtime'] = ad['My.DESIRED_GPURuntime']
+
             # Performance and resource estimates (including JDL magic tweaks)
             origCores = job.get('numberOfCores', 1)
             estimatedMins = int(job['estimatedJobTime'] / 60.0) if job.get('estimatedJobTime') else 12 * 60
@@ -640,7 +662,14 @@ class SimpleCondorPlugin(BasePlugin):
                 ad['My.REQUIRED_ARCH'] = classad.quote(str(requiredArchs))
                 ad['Requirements'] = 'stringListMember(TARGET.Arch, REQUIRED_ARCH)'
 
-            jobParameters.append(ad)    
+            # Inject a microarchitecture classad. If x86_64 not on arch list, return 0
+            if 'X86_64' not in requiredArchs.split(","):
+                ad['My.REQUIRED_MINIMUM_MICROARCH'] = "0"
+            else:
+                minMicroArch = self.tc.getGreaterMicroarchVersionNumber(cmsswVersions, rel_microarchs=rel_microarchs)
+                ad['My.REQUIRED_MINIMUM_MICROARCH'] = str(minMicroArch) 
+
+            jobParameters.append(ad)
              
         return jobParameters
 
