@@ -20,9 +20,12 @@ import urllib.request, urllib.parse, urllib.error
 
 import base64
 import hashlib
+import json
 import logging
 import re
 import time
+import sys
+from pprint import pformat
 from datetime import datetime
 from http.client import HTTPException
 
@@ -133,7 +136,9 @@ class CouchDBRequests(JSONRequests):
                     encode, decode, contentType)
         except HTTPException as e:
             self.checkForCouchError(getattr(e, "status", None),
-                                    getattr(e, "reason", None), data)
+                                    getattr(e, "reason", None),
+                                    data,
+                                    getattr(e, "result", None))
 
         return result
 
@@ -144,27 +149,31 @@ class CouchDBRequests(JSONRequests):
         Check the HTTP status and raise an appropriate exception.
         """
         if status == 400:
-            raise CouchBadRequestError(reason, data, result)
+            raise CouchBadRequestError(reason, data, result, status)
         elif status == 401:
-            raise CouchUnauthorisedError(reason, data, result)
+            raise CouchUnauthorisedError(reason, data, result, status)
         elif status == 403:
-            raise CouchForbidden(reason, data, result)
+            raise CouchForbidden(reason, data, result, status)
         elif status == 404:
-            raise CouchNotFoundError(reason, data, result)
+            raise CouchNotFoundError(reason, data, result, status)
         elif status == 405:
-            raise CouchNotAllowedError(reason, data, result)
+            raise CouchNotAllowedError(reason, data, result, status)
         elif status == 406:
-            raise CouchNotAcceptableError(reason, data, result)
+            raise CouchNotAcceptableError(reason, data, result, status)
         elif status == 409:
-            raise CouchConflictError(reason, data, result)
+            raise CouchConflictError(reason, data, result, status)
+        elif status == 410:
+            raise CouchFeatureGone(reason, data, result, status)
         elif status == 412:
-            raise CouchPreconditionFailedError(reason, data, result)
+            raise CouchPreconditionFailedError(reason, data, result, status)
+        elif status == 413:
+            raise CouchRequestTooLargeError(reason, data, result, status)
         elif status == 416:
-            raise CouchRequestedRangeNotSatisfiableError(reason, data, result)
+            raise CouchRequestedRangeNotSatisfiableError(reason, data, result, status)
         elif status == 417:
-            raise CouchExpectationFailedError(reason, data, result)
+            raise CouchExpectationFailedError(reason, data, result, status)
         elif status == 500:
-            raise CouchInternalServerError(reason, data, result)
+            raise CouchInternalServerError(reason, data, result, status)
         elif status in [502, 503, 504]:
             # There are HTTP errors that CouchDB doesn't raise but can appear
             # in our environment, e.g. behind a proxy. Reraise the HTTPException
@@ -218,6 +227,13 @@ class Database(CouchDBRequests):
                     doc[label] = int(time.time())
         return data
 
+    def getQueueSize(self):
+        """
+        Return the current size of the queue, i.e., how
+        many documents are already queued up
+        """
+        return len(self._queue)
+
     def queue(self, doc, timestamp=False, viewlist=None, callback=None):
         """
         Queue up a doc for bulk insert. If timestamp = True add a timestamp
@@ -231,8 +247,8 @@ class Database(CouchDBRequests):
         if timestamp:
             self.timestamp(doc, timestamp)
         # TODO: Thread this off so that it's non blocking...
-        if len(self._queue) >= self._queue_size:
-            print('queue larger than %s records, committing' % self._queue_size)
+        if self.getQueueSize() >= self._queue_size:
+            logging.warning('queue larger than %s records, committing', self._queue_size)
             self.commit(viewlist=viewlist, callback=callback)
         self._queue.append(doc)
 
@@ -323,30 +339,6 @@ class Database(CouchDBRequests):
             uri += '?' + urllib.parse.urlencode({'rev': rev})
         return Document(id=id, inputDict=self.get(uri))
 
-    def getPrevisousRevision(self, doc_id, numberBefore=1):
-        """
-        :param: doc_id, couch document id
-        :param: numberBefore: previous revision, 1 means one previous revision, 2 means 2 doucments eariler.
-                              special case 0 means the first revision.
-        :return: previous revision document specified by numberBefore
-        """
-        uri = '/%s/%s?revs=true&open_revs=all' % (self.name, urllib.parse.quote_plus(doc_id))
-        revisionRecord = self.get(uri)
-        rev = revisionRecord[0]['ok']['_revisions']
-
-        if numberBefore == 0:
-            revNum = 1
-            revID = rev['start'] - 1
-        else:
-            revNum = rev['start'] - numberBefore
-            revID = rev['ids'][numberBefore]
-        preRev = "%s-%s" % (revNum, revID)
-
-        if doc_id != revisionRecord[0]['ok']['_id']:
-            raise CouchError("revision record doesn match", revisionRecord[0]['ok']['_id'], doc_id)
-
-        return self.document(doc_id, preRev)
-
     def updateDocument(self, doc_id, design, update_func, fields=None, useBody=False):
         """
         Call the update function update_func defined in the design document
@@ -882,7 +874,7 @@ class RotatingDatabase(Database):
         now = datetime.now()
         then = now - self.timing['expire']
 
-        options = {'startkey': 0, 'endkey': time.mktime(then.timetuple())}
+        options = {'startkey': 0, 'endkey': int(time.mktime(then.timetuple()))}
         expired = self._find_dbs_in_state('archived', options)
         for db in expired:
             try:
@@ -894,14 +886,25 @@ class RotatingDatabase(Database):
             self.seed_db.queueDelete(db_state)
         self.seed_db.commit()
 
+    def _create_design_doc(self):
+        """Create a design doc with a view for the rotate state"""
+        tempDesignDoc = {'views': {
+                             'rotateState': {
+                                 'map': "function(doc) {emit(doc.timestamp, doc.rotate_state, doc._id);}"
+                                            },
+                                   }
+                         }
+        self.seed_db.put('/%s/_design/TempDesignDoc' % self.seed_db.name, tempDesignDoc)
+
     def _find_dbs_in_state(self, state, options=None):
+        """Creates a design document with a single (temporary) view in it"""
         options = options or {}
-        # TODO: couchapp this, how to make sure that the app is deployed?
-        find = {'map': "function(doc) {if(doc.rotate_state == '%s') {emit(doc.timestamp, doc._id);}}" % state}
-        uri = '/%s/_temp_view' % self.seed_db.name
-        if options:
-            uri += '?%s' % urllib.parse.urlencode(options)
-        data = self.seed_db.post(uri, find)
+        if self.seed_db.documentExists("_design/TempDesignDoc"):
+            logging.info("Skipping designDoc creation because it already exists!")
+        else:
+            self._create_design_doc()
+
+        data = self.seed_db.loadView("TempDesignDoc", "rotateState", options=options)
         return data['rows']
 
     def inactive_dbs(self):
@@ -916,10 +919,6 @@ class RotatingDatabase(Database):
         """
         return [doc['value'] for doc in self._find_dbs_in_state('archived')]
 
-    def non_rotating_commit(self):
-        # might need this after all....
-        pass
-
     def makeRequest(self, uri=None, data=None, type='GET', incoming_headers=None,
                     encode=True, decode=True, contentType=None,
                     cache=False, rotate=True):
@@ -997,9 +996,13 @@ class CouchServer(CouchDBRequests):
         return Database(dbname=dbname, url=self.url, size=size, ckey=self.ckey, cert=self.cert)
 
     def deleteDatabase(self, dbname):
-        "Delete a database from the server"
+        """Delete a database from the server"""
         check_name(dbname)
         dbname = urllib.parse.quote_plus(dbname)
+        if "cmsweb" in self.url:
+            msg = f"You can't be serious that you want to delete a PRODUCTION database!!! "
+            msg += f"At url: {self.url}, for database name: {dbname}. Bailing out!"
+            raise RuntimeError(msg)
         return self.delete("/%s" % dbname)
 
     def connectDatabase(self, dbname='database', create=True, size=1000):
@@ -1014,18 +1017,11 @@ class CouchServer(CouchDBRequests):
 
     def replicate(self, source, destination, continuous=False,
                   create_target=False, cancel=False, doc_ids=False,
-                  filter=False, query_params=False):
-        """Trigger replication between source and destination. Options are as
-        described in http://wiki.apache.org/couchdb/Replication, in summary:
-            continuous = bool, trigger continuous replication
-            create_target = bool, implicitly create the target database
-            cancel = bool, stop continuous replication
-            doc_ids = list, id's of specific documents you want to replicate
-            filter = string, name of the filter function you want to apply to
-                     the replication, the function should be defined in a design
-                     document in the source database.
-            query_params = dictionary of parameters to pass into the filter
-                     function
+                  filter=False, query_params=False, sleepSecs=0, selector=False):
+        """
+        Trigger replication between source and destination. CouchDB options are
+        defined in: https://docs.couchdb.org/en/3.1.2/api/server/common.html#replicate
+        with further details in: https://docs.couchdb.org/en/stable/replication/replicator.html
 
         Source and destination need to be appropriately urlquoted after the port
         number. E.g. if you have a database with /'s in the name you need to
@@ -1033,14 +1029,29 @@ class CouchServer(CouchDBRequests):
 
         TODO: Improve source/destination handling - can't simply URL quote,
         though, would need to decompose the URL and rebuild it.
+
+        :param source: string with the source url to replicate data from
+        :param destination: string with the destination url to replicate data to
+        :param continuous: boolean to perform a continuous replication or not
+        :param create_target: boolean to create the target database, if non-existent
+        :param cancel: boolean to stop a replication (but we better just delete the doc!)
+        :param doc_ids: a list of specific doc ids that we would like to replicate
+        :param filter: string with the name of the filter function to be used. Note that
+                       this filter is expected to have been defined in the design doc.
+        :param query_params: dictionary of parameters to pass over to the filter function
+        :param sleepSecs: amount of seconds to sleep after the replication job is created
+        :param selector: a new'ish feature for filter functions in Erlang
+        :return: status of the replication creation
         """
-        if source not in self.listDatabases():
+        listDbs = self.listDatabases()
+        if source not in listDbs:
             check_server_url(source)
-        if destination not in self.listDatabases():
+        if destination not in listDbs:
             if create_target and not destination.startswith("http"):
                 check_name(destination)
             else:
                 check_server_url(destination)
+
         if not destination.startswith("http"):
             destination = '%s/%s' % (self.url, destination)
         if not source.startswith("http"):
@@ -1055,7 +1066,12 @@ class CouchServer(CouchDBRequests):
             data["filter"] = filter
             if query_params:
                 data["query_params"] = query_params
-        return self.post('/_replicator', data)
+        if selector: data["selector"] = selector
+
+        resp = self.post('/_replicator', data)
+        # Sleep required for CouchDB 3.x unit tests
+        time.sleep(sleepSecs)
+        return resp
 
     def status(self):
         """
@@ -1088,75 +1104,94 @@ class CouchError(Exception):
         self.status = status
 
     def __str__(self):
-        "Stringify the error"
-        if self.status != None:
-            errorMsg = "NEW ERROR STATUS! UPDATE CMSCOUCH.PY!: %s\n" % self.status
-        else:
-            errorMsg = ""
-        return errorMsg + "%s - reason: %s, data: %s result: %s" % (self.type,
-                                                                    self.reason,
-                                                                    repr(self.data),
-                                                                    self.result)
+        """Stringify the error"""
+        errorMsg = ""
+        if self.type == "CouchError":
+            errorMsg += "A NEW COUCHDB ERROR TYPE/STATUS HAS BEEN FOUND! "
+            errorMsg += "UPDATE CMSCOUCH.PY IMPLEMENTATION WITH A NEW COUCH ERROR/STATUS! "
+            errorMsg += f"Status: {self.status}\n"
+        errorMsg += f"Error type: {self.type}, Status code: {self.status}, "
+        errorMsg += f"Reason: {self.reason}, Data: {repr(self.data)}"
+        return errorMsg
 
 
 class CouchBadRequestError(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchBadRequestError"
 
 
 class CouchUnauthorisedError(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchUnauthorisedError"
 
 
 class CouchNotFoundError(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchNotFoundError"
 
 
 class CouchNotAllowedError(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchNotAllowedError"
 
 class CouchNotAcceptableError(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchNotAcceptableError"
 
 class CouchConflictError(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchConflictError"
 
+
+class CouchFeatureGone(CouchError):
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
+        self.type = "CouchFeatureGone"
+
+
 class CouchPreconditionFailedError(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchPreconditionFailedError"
 
+
+class CouchRequestTooLargeError(CouchError):
+    def __init__(self, reason, data, result, status):
+        # calculate the size of this JSON serialized object
+        docSize = sys.getsizeof(json.dumps(data))
+        errorMsg = f"Document has {docSize} bytes and it's too large to be accepted by CouchDB. "
+        errorMsg += f"Check the CouchDB configuration to see the current value "
+        errorMsg += f"under 'couchdb.max_document_size' (default is 8M bytes)."
+        CouchError.__init__(self, reason, errorMsg, result, status)
+        self.type = "CouchRequestTooLargeError"
+
+
 class CouchExpectationFailedError(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchExpectationFailedError"
 
 class CouchRequestedRangeNotSatisfiableError(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchRequestedRangeNotSatisfiableError"
 
 
 class CouchInternalServerError(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchInternalServerError"
 
 
 class CouchForbidden(CouchError):
-    def __init__(self, reason, data, result):
-        CouchError.__init__(self, reason, data, result)
+    def __init__(self, reason, data, result, status):
+        CouchError.__init__(self, reason, data, result, status)
         self.type = "CouchForbidden"
 
 
@@ -1240,6 +1275,59 @@ class CouchMonitor(object):
             return resp
         return data.get("docs", resp)
 
+    def couchReplicationStatus(self):
+        """
+        check couchdb replication status with compatible output of checkCouchReplications
+
+        :return: a list of dictionaries with the status of the replications and an
+            error message
+        """
+        # Do not import checkStatus at the beginning of the CMSCouch module
+        # to avoid dependecy conflicts in the CMS Sandbox at the WN level
+        from WMCore.Database.CouchMonitoring import checkStatus
+
+        output = []
+        sdict = checkStatus(kind='scheduler')
+        rdict = checkStatus(kind='replicator')
+        method = 'scheduler+replicator'
+        # update sdict only with entries from replicator dict which are not present in scheduler
+        for key, val in rdict['current_status'].items():
+            if key not in sdict['current_status']:
+                sdict['current_status'][key] = val
+        stateFailures = ['error', 'failed']
+        for rid, record in sdict['current_status'].items():
+            if record['state'] in stateFailures:
+                status['state'] = 'error'
+                source = sanitizeURL(record['source'])
+                target = sanitizeURL(record['target'])
+                error = record['error']
+                history = pformat(record['history'])
+                msg = f"Replication from {source} to {target} for document {rid} is in a bad state: {error}; "
+                msg += f"History: {history}"
+                status = {'name': 'CouchServer', 'status': 'error', 'error_message': msg, 'method': method}
+                output.append(status)
+
+        # if our replication is fine we should check that it is not in a stale phase
+        activeTasks = self.getActiveTasks()
+        activeTasks = [task for task in activeTasks if task["type"].lower() == "replication"]
+        resp = self.checkReplicationState()
+        for replTask in activeTasks:
+            if self.isReplicationStale(replTask):
+                source = sanitizeURL(replTask['source'])['url']
+                target = sanitizeURL(replTask['target'])['url']
+                msg = f"Replication from {source} to {target} is stale and it's last"
+                msg += f"update time was at: {replTask.get('updated_on')}"
+                resp['status'] = 'error'
+                resp['error_message'] += msg
+                resp['method'] = 'stale phase'
+                resp['name'] = 'CouchServer'
+                output.append(resp)
+        # check if we did not record any replication status, then add the ok status
+        if len(output) == 0:
+            status = {'name': 'CouchServer', 'status': 'ok', 'error_message': ''}
+            output.append(status)
+        return output
+
     def checkCouchReplications(self, replicationsList):
         """
         Check whether the list of expected replications exist in CouchDB
@@ -1247,9 +1335,11 @@ class CouchMonitor(object):
 
         :param replicationsList: a list of dictionary with the replication
             document setup.
-        :return: a dictionary with the status of the replications and an
+        :return: a list of dictionaries with the status of the replications and an
             error message
         """
+        output = []
+        method = 'comparison of replications docs vs active tasks'
         activeTasks = self.getActiveTasks()
         # filter out any task that is not a database replication
         activeTasks = [task for task in activeTasks if task["type"].lower() == "replication"]
@@ -1258,12 +1348,12 @@ class CouchMonitor(object):
             msg = f"Expected to have {len(replicationsList)} replication tasks, "
             msg += f"but only {len(activeTasks)} in CouchDB. "
             msg += f"Current replications are: {activeTasks}"
-            return {'status': 'error', 'error_message': msg}
+            status = {'name': 'CouchServer', 'status': 'error', 'error_message': msg, 'method': method}
+            output.append(status)
 
         resp = self.checkReplicationState()
         if resp['status'] != 'ok':
-            # then there is a problem, return its status
-            return resp
+            output.append(resp)
 
         # finally, check if replications are being updated in a timely fashion
         for replTask in activeTasks:
@@ -1274,7 +1364,15 @@ class CouchMonitor(object):
                 msg += f"update time was at: {replTask.get('updated_on')}"
                 resp['status'] = 'error'
                 resp['error_message'] += msg
-        return resp
+                resp['method'] = method
+                resp['name'] = 'CouchServer'
+                output.append(resp)
+
+        # check if we did not record any replication status, then add the ok status
+        if len(output) == 0:
+            status = {'name': 'CouchServer', 'status': 'ok', 'error_message': ''}
+            output.append(status)
+        return output
 
     def checkReplicationState(self):
         """
@@ -1313,3 +1411,20 @@ class CouchMonitor(object):
             # then it has been recently updated
             return True
         return False
+
+    def isReplicationStale(self, replInfo, niter=10):
+        """
+        Ensure that the replication document is up-to-date as a
+        function of the checkpoint interval.
+
+        :param replInfo: dictionary with the replication information
+        :param niter: number of iteration for checkpoint interval
+        :return: True if replication is working fine, otherwise False
+        """
+        maxUpdateInterval = niter * replInfo['checkpoint_interval'] / 1000
+        lastUpdate = replInfo["updated_on"]
+
+        if lastUpdate + maxUpdateInterval > int(time.time()):
+            # then it has been recently updated and it means replication is not stale
+            return False
+        return True
