@@ -37,6 +37,50 @@ class StageOutImpl:
         self.numRetries = 3
         self.retryPause = 600
         self.stageIn = stagein
+        self.debuggingTemplate =  "#!/bin/bash\n"
+        self.debuggingTemplate += """
+        echo
+        echo
+        echo "-----------------------------------------------------------"
+        echo "==========================================================="
+        echo
+        echo "Debugging information on failing copy command"
+        echo
+        echo "Current date and time: $(date +"%Y-%m-%d %H:%M:%S")"
+        echo "copy command which failed: {copy_command}"
+        echo "Hostname:   $(hostname -f)"
+        echo "OS:  $(uname -r -s)"
+        echo
+        echo "PYTHON environment variables:"
+        env | grep ^PYTHON
+        echo
+        echo "LD_* environment variables:"
+        env | grep ^LD_
+        echo
+        echo "Information for credentials in the environment"
+        echo "Bearer token content: $BEARER_TOKEN"
+        echo "Bearer token file: $BEARER_TOKEN_FILE"
+        echo
+        echo "Source PFN: {source}"
+        echo "Target PFN: {destination}"
+        echo
+        echo "VOMS proxy info:"
+        voms-proxy-info -all
+        echo "==========================================================="
+        echo "-----------------------------------------------------------"
+        echo
+        """
+
+    def isBearerTokenFileSet(self):
+        """
+        Method to check if the BEARER_TOKEN_FILE environment variable is
+        set and not empty. If so, whether the file exists.
+        :return: True if BEARER_TOKEN_FILE envir var is set and exists, False otherwise.
+        """
+        if os.getenv("BEARER_TOKEN_FILE"):
+            if os.path.exists(os.getenv("BEARER_TOKEN_FILE")):
+                return True
+        return False
 
     @staticmethod
     def splitPFN(pfn):
@@ -132,7 +176,7 @@ class StageOutImpl:
         If no directory is required, do not implement this method
         """
 
-    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None):
+    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
         """
         _createStageOutCommand_
 
@@ -142,13 +186,13 @@ class StageOutImpl:
         """
         raise NotImplementedError("StageOutImpl.createStageOutCommand")
 
-    def createDebuggingCommand(self, sourcePFN, targetPFN, options=None, checksums=None):
+    def createDebuggingCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
         """
         Build a shell command that will report in the logs the details about
         failing stageOut commands
         """
         raise NotImplementedError("StageOutImpl.createDebuggingCommand")
-
+    
     def removeFile(self, pfnToRemove):
         """
         _removeFile_
@@ -178,10 +222,9 @@ class StageOutImpl:
 
         This operator does the actual stage out by invoking the overridden
         plugin methods of the derived object.
-
-
         """
-        #  //
+
+        # //
         # // Generate the source PFN from the plain PFN if needed
         # //
         sourcePFN = self.createSourceName(protocol, inputPFN)
@@ -189,7 +232,7 @@ class StageOutImpl:
         # destination may also need PFN changed
         # i.e. if we are staging in a file from an SE
         targetPFN = self.createTargetName(protocol, targetPFN)
-        #  //
+        # //
         # // Create the output directory if implemented
         # //
         for retryCount in range(self.numRetries + 1):
@@ -203,30 +246,60 @@ class StageOutImpl:
                 msg += "Error details:\n{}\n".format(str(ex))
                 logging.error(msg)
                 if retryCount == self.numRetries:
-                    #  //
+                    # //
                     # // last retry, propagate exception
                     # //
+                    logging.error("Maximum retries exhausted when trying to create the output directory")
                     raise ex
                 time.sleep(self.retryPause)
 
         # //
         # // Create the command to be used.
         # //
-        command = self.createStageOutCommand(sourcePFN, targetPFN, options, checksums)
-        #  //
+        # // This will actually only enforce the definition of the bearer token variables, still x509 auth activated as a backup
+        command = self.createStageOutCommand(sourcePFN, targetPFN, options, checksums, authMethod="TOKEN")
+        # //
         # // Run the command
         # //
 
         stageOutEx = None  # variable to store the possible StageOutError
         for retryCount in range(self.numRetries + 1):
             try:
-                logging.info("Running the stage out...")
+                logging.info("Running the stage out with the available auth method (attempt %d)...", retryCount + 1)
+                logging.info("Command to run: %s", command)
                 self.executeCommand(command)
+                logging.info("\nStage-out succeeded with the current environment.")
                 break
+
             except StageOutError as ex:
-                msg = "Attempt {} to stage out failed.\n".format(retryCount)
+                msg = "Attempt {} to stage out failed with default setup.\n".format(retryCount)
                 msg += "Error details:\n{}\n".format(str(ex))
                 logging.error(msg)
+                logging.info("Retrying with authentication-safe logic...")
+
+                # Authentication-safe fallback logic
+                if os.getenv("X509_USER_PROXY"):
+                    logging.info("Retrying with X509_USER_PROXY with BEARER_TOKEN unset...")
+                    command = self.createStageOutCommand(sourcePFN, targetPFN, options, checksums, authMethod="X509", forceMethod=True)
+                    try:
+                        logging.info("Command to run: %s", command)
+                        self.executeCommand(command)
+                        logging.info("Stage-out succeeded with X509 with BEARER_TOKEN unset.")
+                        return
+                    except StageOutError as fallbackEx:
+                        logging.warning("Fallback with X509_USER_PROXY failed:\n%s", str(fallbackEx))
+
+                if os.getenv("BEARER_TOKEN") or os.getenv("BEARER_TOKEN_FILE"):
+                    logging.info("Retrying with BEARER_TOKEN with X509_USER_PROXY unset...")
+                    command = self.createStageOutCommand(sourcePFN, targetPFN, options, checksums, authMethod="TOKEN", forceMethod=True)
+                    try:
+                        logging.info("Command to run: %s", command)
+                        self.executeCommand(command)
+                        logging.info("Stage-out succeeded with TOKEN with X509_USER_PROXY unset.")
+                        return
+                    except StageOutError as fallbackEx:
+                        logging.warning("Fallback with BEARER_TOKEN failed:\n%s", str(fallbackEx))
+
                 if retryCount == self.numRetries:
                     # Last retry, propagate the information outside of the for loop
                     stageOutEx = ex
@@ -236,6 +309,6 @@ class StageOutImpl:
         # This block will now always be executed after retries are exhausted
         if stageOutEx is not None:
             logging.error("Maximum number of retries exhausted. Further details on the failed command reported below.")
-            command = self.createDebuggingCommand(sourcePFN, targetPFN, options, checksums)
+            command = self.createDebuggingCommand(sourcePFN, targetPFN, options, checksums, authMethod="TOKEN")
             self.executeCommand(command)
             raise stageOutEx from None
